# -*- coding: utf-8 -*-
#
# Copyright © 2008 Ignacio Vazquez-Abrams All rights reserved.
# Copyright © 2008 Red Hat, Inc. All rights reserved.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.  You should have
# received a copy of the GNU General Public License along with this program;
# if not, write to the Free Software Foundation, Inc., 51 Franklin Street,
# Fifth Floor, Boston, MA 02110-1301, USA. Any Red Hat trademarks that are
# incorporated in the source code or documentation are not subject to the GNU
# General Public License and may only be used or replicated with the express
# permission of Red Hat, Inc.
#
# Author(s): Ignacio Vazquez-Abrams <ivazquez@fedoraproject.org>

import turbogears
from turbogears import controllers, expose, paginate, identity, redirect
from turbogears import validate, validators, error_handler, flash
from turbogears.database import session

import cherrypy

from genshi.template.plugin import TextTemplateEnginePlugin
import fas.model.fasmodel as fasmodel
import fas.auth as auth
import fas.sidebar as sidebar
import fas.plugin as plugin

import model

import logging

class ValidFeed(validators.Schema):
    # TODO: Verify that the given URL contains a feed/image
    feed = validators.All(
        validators.String(not_empty=True, max=127),
        validators.Regex(strip=True, regex='^http(s)?://.+')
    )
    name = validators.String(not_empty=True, max=127)
    feed = validators.All(
        validators.String(not_empty=True, max=127),
        validators.Regex(strip=True, regex='^http(s)?://.+')
    )
    hackergotchi = validators.Any(
        validators.All(
            validators.String(not_empty=True, max=127),
            validators.Regex(strip=True, regex='^http(s)?://.+')
        ),
        validators.String(max=0)
    )

class PlanetPlugin(controllers.Controller):
    capabilities = ['planet_plugin']

    def __init__(self):
        '''Create a Planet Controller.'''

    @expose(template="fas.templates.error")
    def error(self, tg_errors=None):
        '''Show a friendly error message'''
        if not tg_errors:
            turbogears.redirect('/')
        return dict(tg_errors=tg_errors)

    @identity.require(turbogears.identity.not_anonymous())
    @expose(template='fas_planet.templates.index')
    def index(self):
        username = identity.current.user_name
        person = fasmodel.People.by_username(username)
        group = fasmodel.Groups.by_name('fedoraplanet')
        feeds = model.Planets.by_person(person)
        canAdd = auth.isApproved(person, group)
        invalid = None
        allentries = None
        if auth.canAdminGroup(person, group):
            allentries = model.ValidEntries.execute().fetchall()
            invalid = model.InvalidEntries.execute().fetchall()
        return dict(canAdd=canAdd, feeds=feeds, invalid=invalid,
            allentries=allentries)

    @identity.require(turbogears.identity.not_anonymous())
    @expose(template='fas_planet.templates.add')
    def add(self):
        username = identity.current.user_name
        person = fasmodel.People.by_username(username)
        return dict(person_id=person.id)
        
    @validate(validators=ValidFeed())
    @error_handler(error)
    @expose(template='fas_planet.templates.add')
    def save(self, person_id, name, feed, hackergotchi):
        entry = model.Planets(person_id=person_id, name=name, feed=feed,
            hackergotchi=hackergotchi)
        flash('Feed successfully added')
        redirect('%sindex' % self.path)

    @identity.require(turbogears.identity.not_anonymous())
    @expose(template='fas_planet.templates.remove')
    def remove(self, feed_id, certain=None):
        username = identity.current.user_name
        person = fasmodel.People.by_username(username)
        group = fasmodel.Groups.by_name('fedoraplanet')
        if feed_id == 'invalid' and not auth.canAdminGroup(person, group):
            flash('You are not authorized to remove other user\'s feeds')
            redirect('%sindex' % self.path)
            return
        else:
            invalid = model.InvalidEntries.execute().fetchall()
            for entry in invalid:
                session.delete(model.Planets.by_id(entry['id']))
            flash('Invalid entries purged')
            redirect('%sindex' % self.path)
        entry = model.Planets.by_id(feed_id)
        if (entry.person_id != person.id and
            not auth.canAdminGroup(person, group)):
            flash('You are not authorized to remove other user\'s feeds')
            redirect('%sindex' % self.path)
            return
        if certain is not None:
            session.delete(entry)
            flash('Feed deleted')
            redirect('%sindex' % self.path)
            return
        return dict(entry=entry)

    @classmethod
    def initPlugin(cls, controller):
        cls.log = logging.getLogger('plugin.fas-planet')
        cls.log.info('Planet plugin initializing')
        try:
            path, self = controller.requestpath(cls, '/planet')
            cls.log.info('Planet plugin hooked')
            self.path = path
            if self.sidebarentries not in sidebar.entryfuncs:
                sidebar.entryfuncs.append(self.sidebarentries)
        except (plugin.BadPathException,
            plugin.PathUnavailableException), e:
            cls.log.info('Planet plugin hook failure: %s' % e)

    @expose('json')
    def list(self):
        entries = []
        for entry in model.ValidEntries.execute().fetchall():
            entries.append(dict(feed=entry.feed, name=entry.name,
                hackergotchi=entry.hackergotchi))
        return dict(entries=entries)

    def delPlugin(self, controller):
        if self.sidebarentries in sidebar.entryfuncs:
            sidebar.entryfuncs.remove(self.sidebarentries)

    def sidebarentries(self):
        entries = []
        if not identity.current.anonymous:
            username = identity.current.user_name
            person = fasmodel.People.by_username(username)
            group = fasmodel.Groups.by_name('fedoraplanet')
            if auth.isApproved(person, group):
                entries.append(('Planet Entries', self.path))
        return entries
